class CamInfo extends ReplicationInfo;

/******************************************************************************
*  Missile Cam. Adds a camera view to rockets and translocator beacons.       *
*  Copyright (C) 2002-2003 Wormbo                                             *
*                                                                             *
*  This program is free software; you can redistribute it and/or modify       *
*  it under the terms of the GNU General Public License as published by       *
*  the Free Software Foundation; either version 2 of the License, or          *
*  (at your option) any later version.                                        *
*                                                                             *
*  This program is distributed in the hope that it will be useful,            *
*  but WITHOUT ANY WARRANTY; without even the implied warranty of             *
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              *
*  GNU General Public License for more details.                               *
*                                                                             *
*  You should have received a copy of the GNU General Public License          *
*  along with this program; if not, write to the Free Software                *
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA  *
******************************************************************************/

var Actor RepCamTarget[5];
var array<Actor> CamTargets;
var MissileCam MyCam;
var PlayerController Controller;
var float CamTimeout;

replication
{
	unreliable if ( Role == ROLE_Authority && bNetOwner && (bNetDirty || bNetInitial) )
		RepCamTarget, Controller;
}

simulated function Tick(float DeltaTime)
{
	local int i, j;
	local bool bFound;
	
	// only execute for local player
	if ( MyCam == None && (Controller == None || Viewport(Controller.Player) == None) )
		return;
	
	// copy new (maybe replicated) targets to dynamic array
	For (i = 0; i < 5; i++) {
		if ( RepCamTarget[i] == None )
			continue;
		bFound = False;
		for (j = 0; j < CamTargets.Length; j++)
			if ( CamTargets[j] == RepCamTarget[i] ) {
				bFound = True;
				break;
			}
		if ( bFound )
			continue;
		CamTargets.Insert(0, 1);
		CamTargets[0] = RepCamTarget[i];
		RepCamTarget[i] = None;
	}
	
	// check for missile cam interaction
	if ( CamTargets.Length > 0 && MyCam == None )
		CreateMissileCam();
	else if ( MyCam != None ) {
		for (i = CamTargets.Length - 1; i >= 0; i--)
			if ( CamTargets[i] == None || CamTargets[i].bDeleteMe
					|| CamTargets[i].IsInState('Dying') )
				CamTargets.Remove(i, 1);
		if ( CamTargets.Length == 0 )
			CamTimeout += DeltaTime;
		else
			CamTimeout = 0;
		if ( CamTimeout > 0.3 )
			MyCam.Remove();
	}
}

simulated function CreateMissileCam()
{
	local Player MyPlayer;
	local Interaction I;
	
	if ( Controller != None )
		MyPlayer = Controller.Player;
	
	if ( MyPlayer != None && MyPlayer.InteractionMaster != None )
		I = MyPlayer.InteractionMaster.AddInteraction("MissileCam.MissileCam", MyPlayer);
	
	if ( MissileCam(I) == None ) {
		if ( I != None && I.Master != None )
			I.Master.RemoveInteraction(I);	// clean up when something went wrong
	}
	else {
		MyCam = MissileCam(I);
		MyCam.CamInfo = Self;
	}
}

simulated function Destroyed()
{
	if ( MyCam != None )
		MyCam.Remove();
	
	Super.Destroyed();
}

defaultproperties
{
}