class MissileCam extends Interaction;

/******************************************************************************
*  Missile Cam. Adds a camera view to rockets and translocator beacons.       *
*  Copyright (C) 2002-2003 Wormbo                                             *
*                                                                             *
*  This program is free software; you can redistribute it and/or modify       *
*  it under the terms of the GNU General Public License as published by       *
*  the Free Software Foundation; either version 2 of the License, or          *
*  (at your option) any later version.                                        *
*                                                                             *
*  This program is distributed in the hope that it will be useful,            *
*  but WITHOUT ANY WARRANTY; without even the implied warranty of             *
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              *
*  GNU General Public License for more details.                               *
*                                                                             *
*  You should have received a copy of the GNU General Public License          *
*  along with this program; if not, write to the Free Software                *
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA  *
******************************************************************************/

#exec TEXTURE IMPORT NAME=MCamFrame FILE=TEXTURES\MCamFrame.dds GROUP="Icons" MIPS=OFF DXT=5 ALPHA=1

struct TexRegion
{
	var() float X;
	var() float Y;
	var() float W;
	var() float H;
	var() Material T;
};

struct DrawRegion
{
	var() float X;
	var() float Y;
	var() float W;
	var() float H;
};

var CamInfo CamInfo;
var Actor Target;
var TexRegion CamBackground, CamFrame, CamTex, CamStatic;
var DrawRegion CamLocation, TextLocation, FrameLocation;

var localized string RocketName, RedeemerName, BeaconName, BeaconStatus, BeaconDisrupted;

function DrawTexRegion(Canvas C, TexRegion TR, DrawRegion DR)
{
	C.SetPos(DR.X * C.ClipX, DR.Y * C.ClipY);
	C.DrawTile(TR.T, DR.W * C.ClipX, DR.H * C.ClipY, TR.X, TR.Y, TR.W, TR.H);
}

function Remove()
{
	Master.RemoveInteraction(Self);
	if ( CamInfo != None )
		CamInfo.MyCam = None;
}

singular function PostRender(Canvas C)
{
	local string TargetName, Description;
	local float XL, YL;
	local byte OldStyle;
	local bool bDrawStatic;
	
	// check if camera info still exists (will not be there when level has changed)
	if ( CamInfo == None || CamInfo.bDeleteMe ) {
		Remove();
		return;
	}
	
	// check that primary cam target isn't the player's viewtarget (switch to secondary cam target then)
	if ( CamInfo.CamTargets.Length > 0 && ViewportOwner.Actor.ViewTarget != CamInfo.CamTargets[0] )
		Target = CamInfo.CamTargets[0];
	else if ( CamInfo.CamTargets.Length > 1 )
		Target = CamInfo.CamTargets[1];
	else
		Target = None;
	
	// check that target still exists (if not, draw static for a short time)
	bDrawStatic = Target == None || Target.bDeleteMe || Target.IsInState('Dying');
	
	OldStyle = C.Style;
	C.Reset();
	if ( !bDrawStatic ) {
		C.DrawPortal(CamLocation.X * C.ClipX, CamLocation.Y * C.ClipY,
				CamLocation.W * C.ClipX, CamLocation.H * C.ClipY,
				Target, GetLocation(Target), GetRotation(Target), 60, True);
		C.Reset();
		
		C.Style = 5;	// STY_Alpha, not available in non-actor classes
		TargetName = GetTargetName(Target);
		Description = GetDescription(Target);
		if ( TargetName != "" ) {
			C.TextSize(TargetName, XL, YL);
			C.SetPos(C.ClipX * TextLocation.X + 0.5 * (C.ClipX * TextLocation.W - XL), C.ClipY * TextLocation.Y);
			C.DrawText(TargetName);
		}
		if ( Description != "" ) {
			C.TextSize(Description, XL, YL);
			C.SetPos(C.ClipX * TextLocation.X + 0.5 * (C.ClipX * TextLocation.W - XL), C.ClipY * (TextLocation.Y + TextLocation.H) - YL);
			C.DrawText(Description);
		}
	}
	else {
		C.Style = 5;	// STY_Alpha, not available in non-actor classes
		DrawTexRegion(C, CamBackground, FrameLocation);
		//C.Style = 4;	// STY_Modulated (the darker, the more transparent)
		DrawTexRegion(C, CamStatic, CamLocation);
	}
	C.Style = 5;	// STY_Alpha (use alpha channel for transparency)
	DrawTexRegion(C, CamFrame, FrameLocation);
	C.Style = OldStyle;
}

event NotifyLevelChange()
{
	Remove();
}

function rotator GetRotation(Actor Other)
{
/*	local vector X, Y, Z;
	
	if ( Other.IsA('TransBeacon') ) {
		if ( Other.Physics != 0 ) {
			LastBeaconDir = Normal(Other.Velocity);
			return rotator(LastBeaconDir);
		}
		else {
			if ( LastBeaconDir == vector(Other.Rotation) )
				return Other.Rotation;
			
			GetAxes(Other.Rotation, X, Y, Z);
			Y = Z Cross Normal(LastBeaconDir);
			X = Y Cross Z;
			
			if ( Y == X ) {
				LastBeaconDir = vector(Other.Rotation);
				return Other.Rotation;
			}
			LastBeaconDir = X;
			return rotator(LastBeaconDir);
		}
	}
*/	
	return rotator(vector(Other.Rotation));
}

function vector GetLocation(Actor Other)
{
	if ( Other.IsA('TransBeacon') )
		return Other.Location + vect(0,0,1) * Other.CollisionHeight;
	return Other.Location;
}

function string GetTargetName(Actor Other)
{
	if ( Other.IsA('RocketProj') )
		return RocketName;
	else if ( Other.IsA('RedeemerProjectile') )
		return RedeemerName;
	else if ( Other.IsA('TransBeacon') )
		return BeaconName;
}

function string GetDescription(Actor Other)
{
	local float Disruption;
	
	if ( Other.IsA('TransBeacon') ) {
		Disruption = TransBeacon(Other).Disruption/TransBeacon(Other).DisruptionThreshold;
		if ( Disruption < 1 )
			return BeaconStatus@(100 - 100 * Disruption)$"%";
		else
			return BeaconStatus@BeaconDisrupted;
	}
}

defaultproperties
{
     bActive=True
     bVisible=True
     bRequiresTick=True
     CamStatic=(T=Material'ScreenNoiseFB',H=96,W=128)
//     CamTex=(T=ScriptedTexture'ScriptedCamTex',H=256,W=256)
     CamBackground=(T=Material'MCamFrame',X=157,Y=4,W=153,H=118)
     CamFrame=(T=Material'MCamFrame',X=3,Y=4,W=153,H=118)
     CamLocation=(X=0.1,Y=0.6,W=0.2,H=0.2)
     TextLocation=(X=0.11,Y=0.61,W=0.18,H=0.18)
     FrameLocation=(X=0.09,Y=0.59,W=0.22,H=0.22)
     RocketName="Rocket"
     RedeemerName="Redeemer"
     BeaconName="Translocator Beacon"
     BeaconStatus="Status:"
     BeaconDisrupted="disrupted"
}