class MutMissileCam extends Mutator;

/******************************************************************************
*  Missile Cam. Adds a camera view to rockets and translocator beacons.       *
*  Copyright (C) 2002-2003 Wormbo                                             *
*                                                                             *
*  This program is free software; you can redistribute it and/or modify       *
*  it under the terms of the GNU General Public License as published by       *
*  the Free Software Foundation; either version 2 of the License, or          *
*  (at your option) any later version.                                        *
*                                                                             *
*  This program is distributed in the hope that it will be useful,            *
*  but WITHOUT ANY WARRANTY; without even the implied warranty of             *
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              *
*  GNU General Public License for more details.                               *
*                                                                             *
*  You should have received a copy of the GNU General Public License          *
*  along with this program; if not, write to the Free Software                *
*  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA  *
******************************************************************************/

var int TargetCount;

function PreBeginPlay()
{
	Super.PreBeginPlay();
	class'RocketProj'.default.bGameRelevant = False;
	class'RedeemerProjectile'.default.bGameRelevant = False;
	class'TransBeacon'.default.bGameRelevant = False;
}

function bool CheckReplacement(Actor Other, out byte bSuperRelevant)
{
	if ( Other.Instigator != None && (Other.IsA('RocketProj')
			|| Other.IsA('RedeemerProjectile') || Other.IsA('TransBeacon')) )
		AddCamViewFor(Other, Other.Instigator);
	
	return Super.CheckReplacement(Other, bSuperRelevant);
}

function ModifyPlayer(Pawn Other)
{
	local CamInfo CamInfo;
	
	if ( PlayerController(Other.Controller) != None ) {	
		ForEach Other.ChildActors(class'CamInfo', CamInfo)
			break;
		if ( CamInfo == None )
			CamInfo = Spawn(class'CamInfo', Other.Controller);
		
		CamInfo.Controller = PlayerController(Other.Controller);
	}
	Super.ModifyPlayer(Other);
}

function AddCamViewFor(Actor Other, Pawn InstigatedBy)
{
	local PlayerController CamOwner;
	local CamInfo CamInfo;
	
	if ( PlayerController(InstigatedBy.Controller) == None )
		return;
	
	CamOwner = PlayerController(InstigatedBy.Controller);
	
	ForEach CamOwner.ChildActors(class'CamInfo', CamInfo)
		break;
	if ( CamInfo == None )
		CamInfo = Spawn(class'CamInfo', CamOwner);
	CamInfo.RepCamTarget[TargetCount++ % 5] = Other;
	CamInfo.Controller = CamOwner;
}

defaultproperties
{
//     ConfigMenuClassName="XInterface.UT2ArenaConfig"
     GroupName="MissileCam"
     FriendlyName="Missile Cam"
     Description="Add a small missile cam view to rocket launcher and redeemer missiles as well as translocator targets."
}